==> tcpl_3.0.0.tar.gz: tcpl/R/tcplFit2.R <==
#' tcpl Wrapper for tcplfit2_core including additional calculations to fit into new schema
#'
#' @param dat output from level 3 processing
#' @param fitmodels list of the models that should be fit with the data
#' @param bmed baseline value, typically should be 0
#'
#' @return Data.table with an additional column fitparams that includes all of the fitting parameters
#' @importFrom tcplfit2 tcplfit2_core
tcplFit2 <- function(dat,
                     fitmodels = c("cnst", "hill", "gnls", "poly1", "poly2", "pow", "exp2", "exp3", "exp4", "exp5"),
                     bmed = NULL) {
  #variable binding
  resp  <-bmad  <-aeid  <-osd  <-m3id <- concentration_unlogged  <-response <- NULL
  # do all the regular fitting things that still need to be done
  res <- dat[, `:=`(c("rmns", "rmds", "nconcs", "med_rmds"), {
    rmns <- mean(resp)
    rmds <- median(resp)
    nconcs <- .N
    med_rmds <- rmds >= (3 * bmad)
    .(rmns, rmds, nconcs, med_rmds)
  }), keyby = .(aeid, spid, logc)][, .(
    bmad = min(bmad), resp_max = max(resp), osd = min(osd), bmed = ifelse(is.null(bmed), 0, max(bmed)),
    resp_min = min(resp), max_mean = max(rmns), max_mean_conc = logc[which.max(rmns)],
    max_med = max(rmds), max_med_conc = logc[which.max(rmds)],
    logc_max = max(logc), logc_min = min(logc), nconc = length(unique(logc)),
    npts = .N, nrep = median(as.numeric(nconcs)), nmed_gtbl = sum(med_rmds) / first(nconcs),
    concentration_unlogged = list(10^(logc)), response = list(resp), m3ids = list(m3id)
  ),
  keyby = .(aeid, spid)
  ][, `:=`(tmpi = seq_len(.N)), keyby = .(aeid)][,
    `:=`(fitparams = list(tcplfit2::tcplfit2_core(unlist(concentration_unlogged),
      unlist(response),
      cutoff = bmad,
      bidirectional = TRUE,
      verbose = FALSE, force.fit = TRUE,
      fitmodels = fitmodels
    ))),
    keyby = .(spid)
  ]

  # return dat with fitted curves in fitparams
  res
}

#' Hitcalling with tcplfit2
#'
#' @param mc4 data.table with level 4 data 
#' @param coff cutoff value for hitcalling
#' 
#'
#' @return Data.table with key value pairs of hitcalling parameters
#' @importFrom dplyr %>% filter group_by summarise left_join inner_join select rowwise mutate pull
#' @importFrom tidyr pivot_longer
#' @importFrom tcplfit2 tcplhit2_core
tcplHit2 <- function(mc4, coff) {
  
  #variable binding
  model <- m4id  <-model_param  <-model_val  <-resp  <- NULL
  params  <-conc  <-bmed  <-onesd  <-df  <-aeid  <- NULL
  fit_method  <-hitcall  <-cutoff  <-top_over_cutoff  <-bmd  <-hit_val <- NULL
  nested_mc4 <- mc4 %>%
    filter(model != "all") %>%
    group_by(m4id) %>%
    summarise(params = list(tcplFit2_nest(data.table(model = model, model_param = model_param, model_val = model_val))))

  # get lvl 3 conc/resp information
  l4_agg <- tcplLoadData(lvl = "agg", fld = "m4id", val = nested_mc4$m4id)
  l3_dat <- l4_agg %>% left_join(tcplLoadData(lvl = 3, fld = "m3id", val = l4_agg$m3id), by = c("aeid", "m3id", "m2id", "m1id", "m0id", "spid", "logc", "resp"))

  # unlog and plug into nested
  nested_mc4 <- nested_mc4 %>% left_join(l3_dat %>% group_by(m4id) %>% summarise(conc = list(10^(logc)), resp = list(resp)), by = "m4id")

  # rejoin the onesd for tcplfit2
  nested_mc4 <- nested_mc4 %>% inner_join(mc4 %>% filter(model_param == "onesd") %>% select(m4id, onesd = model_val), by = "m4id")

  # rejoin for bmed
  nested_mc4 <- nested_mc4 %>% inner_join(mc4 %>% filter(model_param == "bmed") %>% select(m4id, bmed = model_val), by = "m4id")

  # add the cutoff
  # nested_mc4$cutoff <- coff
  # browser()
  test <- nested_mc4 %>%
    dplyr::rowwise() %>%
    mutate(df = list(tcplhit2_core(params = params, conc = unlist(conc), resp = unlist(resp), bmed = bmed, cutoff = coff, onesd = onesd))) %>%
    select(-conc, -resp)
  # test <- NULL
  # for(id in nested_mc4$m4id){
  #   l4_dat <- nested_mc4 %>% filter(m4id == id)
  #   test <- rbind(test,
  #                 tcplhit2_core(params = l4_dat$params[[1]],
  #                 conc = unlist(l4_dat$conc),
  #                 resp = unlist(l4_dat$resp),
  #                 bmed = l4_dat$bmed,
  #                 cutoff = coff,
  #                 onesd = l4_dat$onesd) %>% select(-conc,-resp))
  # }

  res <- cbind(test %>% as.data.table(), test %>% as.data.table() %>% pull(df) %>% rbindlist())

  # mc5 table
  mc5 <- res %>%
    left_join(mc4 %>% select(m4id, aeid) %>% unique(), by = "m4id") %>%
    select(m4id, aeid, modl = fit_method, hitc = hitcall, coff = cutoff) %>%
    mutate(model_type = 2)

  # mc5 param table
  mc5_param <- res %>%
    left_join(mc4 %>% select(m4id, aeid) %>% unique(), by = "m4id") %>%
    select(m4id, aeid, top_over_cutoff:bmd)
  mc5_param <- mc5_param %>%
    tidyr::pivot_longer(cols = top_over_cutoff:bmd, names_to = "hit_param", values_to = "hit_val") %>%
    filter(!is.na(hit_val))

  mc5 %>%
    inner_join(mc5_param, by = c("m4id", "aeid")) %>%
    as.data.table()
}


#' Unnest tcplfit2 parameters into a dataframe
#'
#' @param output list of output from tcplfit2
#'
#' @return list of parameters unnested and compiled into a dataframe
tcplFit2_unnest <- function(output) {
  # unnest for use in invitrodb
  modelnames <- output$modelnames
  for (m in modelnames) {
    assign(m, output[[m]][!names(output[[m]]) %in% c("pars", "sds", "modl")])
  }
  res <- mget(modelnames)

  test <- NULL
  for (m in modelnames) {
    test <- rbind(test, data.frame(model = m, model_param = names(res[[m]]), model_val = unlist(res[[m]]), stringsAsFactors = FALSE, row.names = NULL))
  }
  test
}


#' Nest dataframe into a list that is readable by tcplfit2
#'
#' @param dat a dataframe that has all of the fitting parameters in the style of tcplloaddata
#'
#' @return a list of fitting parameters that can be consumed by tcplfit2
tcplFit2_nest <- function(dat) {
  # renest
  modelnames <- unique(dat$model)
  for (m in modelnames) {
    assign(m, split(dat[dat$model == m, ]$model_val, dat[dat$model == m, ]$model_param))
  }
  
  # each model requires the pars parameter for tcplhit2_core
  for (m in modelnames) {
    if(m == "cnst") modpars <-  "er"
    if(m == "exp2") modpars <- paste0(c("a", "b", "er"))
    if(m == "exp3") modpars <- paste0(c("a", "b", "p", "er"))
    if(m == "exp4") modpars <- paste0(c("tp", "ga", "er"))
    if(m == "exp5") modpars <- paste0(c("tp", "ga", "p", "er"))
    if(m == "hill") modpars <- paste0(c("tp", "ga", "p", "er"))
    if(m == "poly1") modpars <- paste0(c("a", "er"))
    if(m == "poly2") modpars <- paste0(c("a", "b", "er"))
    if(m == "pow") modpars <- paste0(c("a", "p", "er"))
    if(m == "gnls") modpars <- paste0(c("tp", "ga", "p", "la", "q", "er"))
    assign(m,append(get(m),list(pars = modpars)))
  }

  out1 <- c(
    mget(modelnames),
    list(modelnames = modelnames)
  )
}

log_model_params <- function(datc) {
  datc %>%
    mutate(model_val = case_when(
      model_param == "ac50" ~ log10(model_val),
      TRUE ~ model_val
    )) %>%
    as.data.table()
}

unlog_model_params <- function(datc) {
  datc %>%
    mutate(model_val = case_when(
      model_param == "ac50" ~ 10^(model_val),
      TRUE ~ model_val
    )) %>%
    as.data.table()
}

==> tcpl_3.0.1.tar.gz: tcpl/R/tcplFit2.R <==
#' tcpl Wrapper for tcplfit2_core including additional calculations to fit into new schema
#'
#' @param dat output from level 3 processing
#' @param fitmodels list of the models that should be fit with the data
#' @param bmed baseline value, typically should be 0
#' @param bidirectional boolean, default is TRUE (bidirectional fitting)
#'
#' @return Data.table with an additional column fitparams that includes all of the fitting parameters
#' @importFrom tcplfit2 tcplfit2_core
tcplFit2 <- function(dat,
                     fitmodels = c("cnst", "hill", "gnls", "poly1", "poly2", "pow", "exp2", "exp3", "exp4", "exp5"),
                     bmed = NULL,
                     bidirectional = TRUE) {
  #variable binding
  resp  <-bmad  <-aeid  <-osd  <-m3id <- concentration_unlogged  <-response <- NULL
  # do all the regular fitting things that still need to be done
  res <- dat[, `:=`(c("rmns", "rmds", "nconcs", "med_rmds"), {
    rmns <- mean(resp)
    rmds <- median(resp)
    nconcs <- .N
    med_rmds <- rmds >= (3 * bmad)
    .(rmns, rmds, nconcs, med_rmds)
  }), keyby = .(aeid, spid, logc)][, .(
    bmad = min(bmad), resp_max = max(resp), osd = min(osd), bmed = ifelse(is.null(bmed), 0, max(bmed)),
    resp_min = min(resp), max_mean = max(rmns), max_mean_conc = logc[which.max(rmns)],
    max_med = max(rmds), max_med_conc = logc[which.max(rmds)],
    logc_max = max(logc), logc_min = min(logc), nconc = length(unique(logc)),
    npts = .N, nrep = median(as.numeric(nconcs)), nmed_gtbl = sum(med_rmds) / first(nconcs),
    concentration_unlogged = list(10^(logc)), response = list(resp), m3ids = list(m3id)
  ),
  keyby = .(aeid, spid)
  ][, `:=`(tmpi = seq_len(.N)), keyby = .(aeid)][,
    `:=`(fitparams = list(tcplfit2::tcplfit2_core(unlist(concentration_unlogged),
      unlist(response),
      cutoff = bmad,
      bidirectional = bidirectional,
      verbose = FALSE, force.fit = TRUE,
      fitmodels = fitmodels
    ))),
    keyby = .(spid)
  ]

  # return dat with fitted curves in fitparams
  res
}

#' Hitcalling with tcplfit2
#'
#' @param mc4 data.table with level 4 data 
#' @param coff cutoff value for hitcalling
#' 
#'
#' @return Data.table with key value pairs of hitcalling parameters
#' @importFrom dplyr %>% filter group_by summarise left_join inner_join select rowwise mutate pull
#' @importFrom tidyr pivot_longer
#' @importFrom tcplfit2 tcplhit2_core
tcplHit2 <- function(mc4, coff) {
  
  #variable binding
  model <- m4id  <-model_param  <-model_val  <-resp  <- NULL
  params  <-conc  <-bmed  <-onesd  <-df  <-aeid  <- NULL
  fit_method  <-hitcall  <-cutoff  <-top_over_cutoff  <-bmd  <-hit_val <- NULL
  nested_mc4 <- mc4 %>%
    filter(model != "all") %>%
    group_by(m4id) %>%
    summarise(params = list(tcplFit2_nest(data.table(model = model, model_param = model_param, model_val = model_val))))

  # get lvl 3 conc/resp information
  l4_agg <- tcplLoadData(lvl = "agg", fld = "m4id", val = nested_mc4$m4id)
  l3_dat <- l4_agg %>% left_join(tcplLoadData(lvl = 3, fld = "m3id", val = l4_agg$m3id), by = c("aeid", "m3id", "m2id", "m1id", "m0id", "spid", "logc", "resp"))

  # unlog and plug into nested
  nested_mc4 <- nested_mc4 %>% left_join(l3_dat %>% group_by(m4id) %>% summarise(conc = list(10^(logc)), resp = list(resp)), by = "m4id")

  # rejoin the onesd for tcplfit2
  nested_mc4 <- nested_mc4 %>% inner_join(mc4 %>% filter(model_param == "onesd") %>% select(m4id, onesd = model_val), by = "m4id")

  # rejoin for bmed
  nested_mc4 <- nested_mc4 %>% inner_join(mc4 %>% filter(model_param == "bmed") %>% select(m4id, bmed = model_val), by = "m4id")

  # add the cutoff
  # nested_mc4$cutoff <- coff
  # browser()
  test <- nested_mc4 %>%
    dplyr::rowwise() %>%
    mutate(df = list(tcplhit2_core(params = params, conc = unlist(conc), resp = unlist(resp), bmed = bmed, cutoff = coff, onesd = onesd))) %>%
    select(-conc, -resp)
  # test <- NULL
  # for(id in nested_mc4$m4id){
  #   l4_dat <- nested_mc4 %>% filter(m4id == id)
  #   test <- rbind(test,
  #                 tcplhit2_core(params = l4_dat$params[[1]],
  #                 conc = unlist(l4_dat$conc),
  #                 resp = unlist(l4_dat$resp),
  #                 bmed = l4_dat$bmed,
  #                 cutoff = coff,
  #                 onesd = l4_dat$onesd) %>% select(-conc,-resp))
  # }

  res <- cbind(test %>% as.data.table(), test %>% as.data.table() %>% pull(df) %>% rbindlist())

  # mc5 table
  mc5 <- res %>%
    left_join(mc4 %>% select(m4id, aeid) %>% unique(), by = "m4id") %>%
    select(m4id, aeid, modl = fit_method, hitc = hitcall, coff = cutoff) %>%
    mutate(model_type = 2)

  # mc5 param table
  mc5_param <- res %>%
    left_join(mc4 %>% select(m4id, aeid) %>% unique(), by = "m4id") %>%
    select(m4id, aeid, top_over_cutoff:bmd)
  mc5_param <- mc5_param %>%
    tidyr::pivot_longer(cols = top_over_cutoff:bmd, names_to = "hit_param", values_to = "hit_val") %>%
    filter(!is.na(hit_val))

  mc5 %>%
    inner_join(mc5_param, by = c("m4id", "aeid")) %>%
    as.data.table()
}


#' Unnest tcplfit2 parameters into a dataframe
#'
#' @param output list of output from tcplfit2
#'
#' @return list of parameters unnested and compiled into a dataframe
tcplFit2_unnest <- function(output) {
  # unnest for use in invitrodb
  modelnames <- output$modelnames
  for (m in modelnames) {
    assign(m, output[[m]][!names(output[[m]]) %in% c("pars", "sds", "modl")])
  }
  res <- mget(modelnames)

  test <- NULL
  for (m in modelnames) {
    lst <- lapply(res[[m]], function(x){ if(length(x) < 1) { x <- NA }; x })
    test <- rbind(test, data.frame(model = m, model_param = names(lst), model_val = unlist(lst), stringsAsFactors = FALSE, row.names = NULL))
  }
  test
}


#' Nest dataframe into a list that is readable by tcplfit2
#'
#' @param dat a dataframe that has all of the fitting parameters in the style of tcplloaddata
#'
#' @return a list of fitting parameters that can be consumed by tcplfit2
tcplFit2_nest <- function(dat) {
  # renest
  modelnames <- unique(dat$model)
  for (m in modelnames) {
    assign(m, split(dat[dat$model == m, ]$model_val, dat[dat$model == m, ]$model_param))
  }
  
  # each model requires the pars parameter for tcplhit2_core
  for (m in modelnames) {
    if(m == "cnst") modpars <-  "er"
    if(m == "exp2") modpars <- paste0(c("a", "b", "er"))
    if(m == "exp3") modpars <- paste0(c("a", "b", "p", "er"))
    if(m == "exp4") modpars <- paste0(c("tp", "ga", "er"))
    if(m == "exp5") modpars <- paste0(c("tp", "ga", "p", "er"))
    if(m == "hill") modpars <- paste0(c("tp", "ga", "p", "er"))
    if(m == "poly1") modpars <- paste0(c("a", "er"))
    if(m == "poly2") modpars <- paste0(c("a", "b", "er"))
    if(m == "pow") modpars <- paste0(c("a", "p", "er"))
    if(m == "gnls") modpars <- paste0(c("tp", "ga", "p", "la", "q", "er"))
    assign(m,append(get(m),list(pars = modpars)))
  }

  out1 <- c(
    mget(modelnames),
    list(modelnames = modelnames)
  )
}

log_model_params <- function(datc) {
  datc %>%
    mutate(model_val = case_when(
      model_param == "ac50" ~ log10(model_val),
      TRUE ~ model_val
    )) %>%
    as.data.table()
}

unlog_model_params <- function(datc) {
  datc %>%
    mutate(model_val = case_when(
      model_param == "ac50" ~ 10^(model_val),
      TRUE ~ model_val
    )) %>%
    as.data.table()
}
